/*
 * Licensed Materials - Property of IBM
 * 5725-I43 (C) Copyright IBM Corp. 2014. All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
var jwt = require("jsonwebtoken");

module.exports = function(cert, requiredScope) {

	function err(desc,code,status,tokenScope) {
		var errObj = {err:code,errMsg:desc,httpStatus:status};
		// WWW-Authenticate: Bearer scope=<scope>[,error=<error_code>][,error_description=<description>] 
		var headerScope = requiredScope;
		if (!headerScope) {
//			console.log("Using token scope " + tokenScope);
			headerScope = tokenScope;
		}
		if (!headerScope) {
			headerScope = "wl-default-scope";
		}
		var header = "Bearer ";
		header = header + "scope=" + headerScope + ",";
		header = header + "error="+code+",error_description="+desc;
		errObj.wwwAuthenticateHeader = header;
//		console.log(errObj);
		return errObj;
	}

	function validateToken(token, callback) {
		if (!token) {
			callback(err("Missing access token","missing_token",401), null);
		} else {
			jwt.verify(token, cert, function(error, payload) {
				if (error) {
					callback(err(error,"invalid_token",401), null);
				} else {
					try {
						var value = JSON.parse(payload);
//						console.log(value);
						if (value.version != "WL1.0") {
							callback(err("Unrecognized token type, only WL1.0 tokens are supported","invalid_token",401,value.scope), null);
						} else if (value.expiration && Date.now() > value.expiration) {
							callback(err("Token has expired","invalid_token",401, value.scope), null);
						} else if (requiredScope && value.scope != requiredScope) {
							callback(err("Required scope is missing","insufficient_scope",403), null);
						} else {
							// Token is valid
							callback(null, value);
						}
					} catch (e) {
						callback(err("Unable to parse payload " + e,"invalid_token",401), null);
					}
				}
			});
		}
	}

	return {
		validate : validateToken
	}
}
